%% function [n_partitions, partitions_v] = find_partitions(J_k1,trellis_paths,...
%    trellis_path_index,trellis_weights,n_paths,max_partitions)
% Function to find most likely partitions using a greedy approach
% The function inputs are as follow:
% J_k1: number of Gaussian components in predicted PHD
% trellis_paths: the collection of best observation subsets for each of the J_k1 Gaussian components
% trellis_path_index: the index of paths for each component in the collection all_paths
% trellis_weights: the path weight dW without the clutter term
% n_paths: number of paths in all_paths
% max_partitions: maximum number of partitions for the greedy algorithm
% The function outputs are:
% n_partitions: The total number of partitions generated by the greedy algorithm
% partitions_v: A binary vector representation of the partitions generated using the collection all_paths

function [n_partitions, partitions_v] = find_partitions(J_k1,trellis_paths,...
    trellis_path_index,trellis_weights,n_paths,max_partitions)

n_partitions = 1;
partitions_list{1} = [];
unique_partitions = zeros(1,J_k1);
w_partitions = 1;

for r = 1:J_k1
    n_temp = 0; w_temp = []; temp_list = {}; temp_up = [];
    for i = 1:n_partitions
        for l = 1:size(trellis_paths{r},1)
            if ~overlap(partitions_list{i},trellis_paths{r}(l,:))
                n_temp = n_temp + 1;
                temp_up(n_temp,:) = unique_partitions(i,:);
                if trellis_path_index{r}(l) ~= 0 % if not the empty path
                    temp_list{n_temp} = [partitions_list{i}; trellis_paths{r}(l,:)];
                    temp_up(n_temp,r) = trellis_path_index{r}(l);
                    
                    w_temp(n_temp) = w_partitions(i)*trellis_weights{r}(l);
                else
                    temp_list{n_temp} = partitions_list{i};
                    
                    w_temp(n_temp) = w_partitions(i);
                end
            end
        end
    end
    % keep only unique partitions
    temp_up = sort(temp_up,2,'descend');
    [temp_up,index,~] = unique(temp_up,'rows');
    temp_list = temp_list(index);
    w_temp = w_temp(index);
    n_temp = size(index,1);
    
    % keep partitions with highest weights
    n_partitions = min(max_partitions,n_temp);
    [~, index] = sort(w_temp,'descend');
    partitions_list = temp_list(index(1:n_partitions));
    unique_partitions = temp_up(index(1:n_partitions),:);
    w_partitions = w_temp(index(1:n_partitions));
end

% A binary vector representation of the partitions
partitions_v = zeros(n_partitions,n_paths);
for i = 1:n_partitions
    partitions_v(i,unique_partitions(i,unique_partitions(i,:)~=0)) = 1;
end

end


% function to find if the observation subset 'path' overlaps with the 'partition'
function does_overlap = overlap(partition,path)

does_overlap = 0;

p_size = size(partition,1);
no_sensors = size(path,2);

p = 0;
while ~does_overlap && p < p_size
    p = p + 1;
    temp = partition(p,:);
    for s = 1:no_sensors
        if (temp(s) ~= -1) && (path(s) ~= -1) && (temp(s) == path(s))
            does_overlap = 1;
            break;
        end
    end
end

end
