function [w_k_new, m_k_new, P_k_new, J_k_new] = prune_merge(w_threshold,...
    merge_threshold, J_max, w_k, m_k, P_k)

% Prune & merge Gaussian components
% 1. Prune components with weights less than w_threshold
% 2. Merge remaining components that are within the distance of merge_threshold
% 3. Report only the J_max highest-weighted components after merging
% Based on Tables II and III in Vo&Ma 2006 paper.

ind = find(w_k>w_threshold);
merged = 0;
not_merged = ind;
ndim = size(m_k,1);
w_merge = []; m_merge = []; P_merge = [];
while ~isempty(not_merged)
    [max_w, ind_max] = max(w_k(not_merged));
    ind_max = not_merged(ind_max);
    ind_rest = not_merged(~(not_merged==ind_max));
    ind_merged = ind_max;
    
    P_to_invert = P_k(:,:,ind_max);
    symmetricP = mean(cat(3,P_to_invert,P_to_invert'),3);
    cholP= chol(symmetricP);
    cholInvP =  cholP \ eye(size(cholP));
    inverseP = cholInvP*cholInvP';
    
    m_minus = bsxfun(@minus,m_k(:,ind_max),m_k(:,ind_rest));
    mTPm = sum(m_minus'.*(inverseP*m_minus)',2); 
    % dot product above only computes the diagonal entries
    ind_merged = horzcat(ind_merged,ind_rest(mTPm<=merge_threshold)); 
    
    merged = merged+1;
    w_merge(merged) = sum(w_k(ind_merged));
    m_merge(:,merged) = m_k(:,ind_merged)*w_k(ind_merged)'./w_merge(merged);
    m_diff = bsxfun(@minus,m_merge(:,merged), m_k(:,ind_merged));
    P_merge(:,:,merged) = zeros(ndim,ndim);
    for j = 1:length(ind_merged)
        P_merge(:,:,merged) = P_merge(:,:,merged) + ...
            ((P_k(:,:,ind_merged(j))+m_diff(:,j)*m_diff(:,j)') ...
            * w_k(ind_merged(:,j)));
    end
    P_merge(:,:,merged) = P_merge(:,:,merged)./w_merge(merged);
    not_merged = not_merged(~ismembc(not_merged,sort(ind_merged)));
end

% sort and prepare output
J = min(length(w_merge),J_max);
[~, sorted] = sort(w_merge,'descend');

w_k_new = w_merge(sorted(1:J));
m_k_new = m_merge(:,sorted(1:J));
P_k_new = P_merge(:,:,sorted(1:J));
J_k_new = J;

end
